#!/usr/bin/python
import os.path
import sys
import subprocess
import uuid
import pwd
import grp

from charmhelpers.core.host import (
    service_start,
    service_stop,
    adduser,
    log,
    mkdir,
    symlink,
)

from charmhelpers.fetch import (
    apt_install,
    apt_update,
)

from charmhelpers.core.hookenv import (
    Hooks,
    relation_get,
    relation_set,
    relation_ids,
    related_units,
    config,
    execution_environment,
)

hooks = Hooks()

required_pkgs = [
    'nodejs',
    'curl',
    'bzr',
    'daemon',
    'abiword',
    'npm',
]

APP_DIR = str(config("install_path"))
APP_NAME = str(config("application_name"))
APP_USER = str(config("application_user"))
APP_URL = str(config("application_url"))
APP_REVNO = str(config("application_revision"))

def write_file(path, fmtstr, owner='root', group='root', perms=0444, context=None):
    """Create or overwrite a file with the contents of a string"""
    if context == None:
        context = {}
    else:
        context = dict(context)
        context.update(execution_environment())
    log("Writing file {} {}:{} {:o}".format(path, owner, group,
        perms))
    uid = pwd.getpwnam(owner.format(**context)).pw_uid
    gid = grp.getgrnam(group.format(**context)).gr_gid
    with open(path.format(**context), 'w') as target:
        os.fchown(target.fileno(), uid, gid)
        os.fchmod(target.fileno(), perms)
        target.write(fmtstr.format(**context))

def render_template_file(source, destination, **context):
    """Create or overwrite a file using a template"""
    log("Rendering template {} for {}".format(source,
        destination))
    with open(source, 'r') as template:
        write_file(destination, template.read(),
            context=context)

def add_extra_repos():
    extra_repos = config('extra_archives')
    if not extra_repos:
        return

    for repo in set(extra_repos.split()):
        subprocess.check_call(['add-apt-repository', '--yes', repo])

def start():
    subprocess.check_call(['open-port','9001'])
    service_start(APP_NAME)

def stop():
    service_stop(APP_NAME)

def configure_dirty():
    log("Configuring {} with local dirty database.".format(APP_NAME))
    stop()
    render_template_file("templates/settings.json.dirty", "{}/settings.json".format(APP_DIR),
        APP_DIR=APP_DIR)
    mkdir("{}-db/".format(APP_DIR), APP_USER, APP_USER, 0700)
    start()

def configure_mysql():
    log("Configuring {} with mysql database.".format(APP_NAME))
    host = relation_get("host")
    if not host:
        return
    user = relation_get("user")
    password = relation_get("password")
    database = relation_get("database")
    stop()
    render_template_file("templates/settings.json.mysql", "{}/settings.json".format(APP_DIR),
        host=host, user=user, password=password, database=database)
    start()

@hooks.hook("install")
def install():
    log("Installing {}".format(APP_NAME))
    add_extra_repos()
    apt_update()
    apt_install(required_pkgs, options=['--force-yes'])
    adduser(APP_USER, str(uuid.uuid4()))
    installdir = APP_DIR+"."+str(uuid.uuid4())
    subprocess.check_call(['bzr', 'branch', '-r', APP_REVNO, APP_URL, installdir])
    write_file("{}/APIKEY.txt".format(installdir), str(uuid.uuid4()), APP_USER, APP_USER, 0600)
    write_file("{}/src/.ep_initialized".format(installdir), "", APP_USER, APP_USER, 0600)
    stop()
    if os.path.exists(APP_DIR):
        os.unlink(APP_DIR)
    symlink(installdir, APP_DIR)
    mkdir("{}/var".format(APP_DIR), APP_USER, APP_USER, 0700)
    units = relation_ids("db")
    if not units:
        configure_dirty()
    else:
        configure_mysql()
    start()

@hooks.hook("config-changed")
def config_change():
    log("Installing upstart configuration for {}".format(APP_NAME))
    render_template_file('templates/etherpad-lite.conf', '/etc/init/etherpad-lite.conf',
         APP_NAME=APP_NAME, APP_USER=APP_USER, APP_DIR=APP_DIR)
    configure_dirty()
    stop()
    start()

@hooks.hook("upgrade-charm")
def upgrade_charm():
    log("Upgrading charm for {}".format(APP_NAME))
    install()

@hooks.hook("db-relation-joined")
def db_relation_joined():
    configure_mysql()

@hooks.hook("db-relation-changed")
def db_relation_changed():
    configure_mysql()

@hooks.hook("db-relation-broken")
def db_relation_broken():
    configure_dirty()

@hooks.hook("pgsql-relation-joined")
def pgsql_relation_joined():
    configure_pgsql()

@hooks.hook("pgsql-relation-changed")
def pgsql_relation_changed():
    configure_pgsql()

@hooks.hook("pgsql-relation-broken")
def pgsql_relation_broken():
    configure_dirty()

@hooks.hook("website-relation-joined")
def website_relation_joined():
    log("Adding website relation for {}".format(APP_NAME))
    relation_set(port="9001", hostname=subprocess.check_output(["unit-get","private-address"]))

if __name__ == "__main__":
    hooks.execute(sys.argv)
